@@ -288,7 +288,9 @@ def isoftype(x, xtype):
     if xtype == type: return isatype(x)
     if xtype is None: return True
     if not iterable(xtype):
-        if isinstance(xtype, Type): return xtype(x) not in (None, False)
+        if isinstance(xtype, Type):
+            if xtype.isunion and type(x) in xtype.types: return True
+            return xtype(x) not in (None, False)
         if isinstance(xtype, type): return isinstance(x, xtype)
         if callable(xtype):
             name = _get_func_name(xtype)
@@ -298,7 +300,6 @@ def isoftype(x, xtype):
                 try: return xtype(x) not in (None, False)
                 except: pass
             return False
-        raise SystemError("Conflict between functions 'isatype' and 'isoftype', please contact the developer for more information. ")
     if type(x) in xtype: return True
     for xt in xtype:
         if isoftype(x, xt): return True
@@ -379,7 +380,6 @@ class Type(type):
         return output
 
     def __new__(cls, *_T, name=None, shape=tuple(), inv=False, ext=False, itypes=None):
-        nT = len(_T)
         _T = Type.extractType(_T)
 
         self = super().__new__(cls, cls.__name__ + '.' + '/'.join(map(typename, _T)), tuple(t for t in _T if inheritable(t)), {})
@@ -396,18 +396,26 @@ class Type(type):
     def __getitem__(self, i=None):
         if i == 0 or i is None: i = tuple()
         if isoftype(i, sequence): pass
-        elif isatype(i): return self@i
         elif isinstance(i, slice): return self@{i.start:i.stop}
-        elif isoftype(i, int): i = (i,)
+        elif isinstance(i, int): i = (i,)
+        elif isatype(i): return self@i
         else: raise TypeHintError("Wrong size specifier. ")
-        if all([x is None or isitertype(x) for x in self.types]):
+        if all(x is None or isitertype(x) for x in self.types):
             if not all([isoftype(x, int) for x in i]):
                 if isinstance(i, list): return self@i
                 if all([isinstance(x, slice) for x in i]): return self@{x.start:x.stop for x in i}
                 if all([isinstance(x, str) or isatype(x) for x in i]): return self@Type(*i)
                 raise TypeHintError("Wrong size specifier. ")
-            return Type(self.types, name=self.name, shape=i, inv=self.inv, ext=self.extendable, itypes=self.itemtypes)
-        else: return Type(sequence, name=f"Iterable", shape=i, inv=self.inv, ext=self.extendable, itypes=self)
+            T = deepcopy(self)
+            T.shape = i
+            return T
+        else:
+            T = deepcopy(self)
+            T.types = sequence
+            T.name = "Iterable"
+            T.shape = i
+            T.itemtypes = self
+            return T
 
     def __lshift__(self, i):
         if all([isofsubclass(x, dict) for x in self.types]) and len(i) == 2: return self@{i[0]: i[1]}
@@ -427,23 +435,29 @@ class Type(type):
 
     def __matmul__(self, other):
         if other is None: return
-        if not isitertype(self): raise TypeError("Only iterable Type can use @ to specify the items. ")
-        if isinstance(other, Type):
-            return Type(self.types, name=self.name, shape=self.shape, inv=self.inv, ext=self.extendable, itypes=other.itemtypes if iterable(other) else other)
-        if all([isarray(x) for x in self.types]) and isdtype(other):
-            return Type(self.types, name=self.name, shape=self.shape, inv=self.inv, ext=self.extendable, itypes=other)
+        if not all(isitertype(x) for x in self.types): raise TypeError("Only iterable Type can use @ to specify the items. ")
+        if isinstance(other, Type) or all(isarray(x) for x in self.types) and isdtype(other):
+            T = deepcopy(self)
+            T.itemtypes = other
+            return T
         shape = self.shape
         if len(shape) == 0 and isoftype(other, (list, tuple)) and len(other) > 0: shape = [len(other)]
         if not iterable(other): other = (other,)
         if len(shape) == 1 and len(other) not in (1, shape[0]): raise TypeHintError("Wrong number of item types for @. ")
         if isinstance(other, dict) and not all([isofsubclass(x, dict) for x in self.types]): raise TypeHintError("Please do use {a:b} typehint for dicts.")
-        return Type(self.types, name=self.name, shape=shape, inv=self.inv, ext=self.extendable, itypes=other)
+        T = deepcopy(self)
+        T.itemtypes = other
+        return T
 
     def __pos__(self):
-        return Type(self.types, name=self.name, shape=self.shape, inv=self.inv, ext=True, itypes=self.itemtypes)
+        T = deepcopy(self)
+        T.extendable = True
+        return T
 
     def __invert__(self):
-        return Type(self.types, name=self.name, shape=self.shape, inv=not self.inv, ext=self.extendable, itypes=self.itemtypes)
+        T = deepcopy(self)
+        T.inv = not self.inv
+        return T
 
     def __str__(self):
         string = f"<Type '{'*' * self.extendable}{'non-' * self.inv}{self.rawname}"
@@ -490,10 +504,11 @@ class Type(type):
         true = not self.inv
         false = self.inv
 
+        if self.isunion: return isoftype(arg, self.types)
         if isoftype(arg, self.types):
             if len(self.shape) == 0: pass
             elif len(self.shape) == 1 and (len(arg) == self.shape[0] or self.shape[0] in (None, -1)): pass
-            elif len(self.shape) > 1 and hasattr(arg, 'shape') and \
+            elif len(self.shape) > 1 and isarray(arg) and \
                 all([a==b or b in (None, -1) for a, b in zip(arg.shape, self.shape)]): pass
             else: return false
             if self.itemtypes != None:
@@ -523,9 +538,7 @@ class Type(type):
                     return false
                 elif isatype(self.itemtypes):
                     import random
-                    for _ in range(min(len(arg), 5)):
-                        try: x = random.choice(arg)
-                        except: x = arg.pop()
+                    for x in random.sample(list(arg), min(len(arg), 5)):
                         if not isoftype(x, self.itemtypes): return false
                 else: raise TypeHintError("Invalid item types after @, please use iterable or types.")
             return true
